/************************************************************************
* $Id: ColorMaskFrame.java,v 1.10 1998/07/24 05:17:43 jjung Exp $
*
* ColorMaskFrame - to allow a interactive methoid for coloring/masking
*	objects by their current value
*
* version 1.0 beta
* written by Bradley Huffaker (04/17/98)
* modified by Jaeyeon Jung (04/24/98)
*
*************************************************************************
*************************************************************************
By accessing this software, COLORMASKFRAME, you are duly informed of and 
agree to be bound by the conditions described below in this notice:
 
This software product, COLORMASKFRAME, is developed by Bradley Huffaker, and
Jaeyeon Jung copyrighted(C) 1998 by the University of California, 
San Diego (UCSD), with all rights reserved.  UCSD administers the NLANR 
Cache grants, NCR-9796082 and NCR-9616602, under which most of this code 
was developed.
 
There is no charge for COLORMASKFRAME software. You can redistribute it and/or
modify it under the terms of the GNU General Public License, v. 2 dated
June 1991 which is incorporated by reference herein.  COLORMASKFRAME is
distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use of
it will not infringe on any third party's intellectual property
rights.
 
You should have received a copy of the GNU GPL along with the COLORMASKFRAME
program.  Copies can also be obtained from
http://www.gnu.org/copyleft/gpl.html or by writing to
 
                        University of California, San Diego
                        SDSC/CAIDA/NLANR
                        9500 Gilman Dr., MS-0505
                        La Jolla, CA 92093 - 0505  USA
 
  Or contact INFO@NLANR.NET
***********************************************************************/
package bhuffake.plankton;
import bhuffake.tools.GridPanel;
import bhuffake.tools.Strings;
import java.awt.*;
import java.lang.Math;

public class ColorMaskFrame extends Frame
{
	protected DList dlist;
	private SmallDisplay sDisplay;
	private Scrollbar hbar, vbar;

	// Controlling Buttons/Choices
	Choice Process = new Choice();
	final String PROCESS_COLOR = "Color objects where values";
	final int COLOR = 0;
	final String PROCESS_MASK = "Show only objects where values";
	final int MASK = 1;

	// Controls if this selects objects
	Checkbox SelectRoot = new Checkbox("Select object as root");

	Choice Control = new Choice();
	final String CONTROL_EQUAL =		"=";
	final int EQUAL = 0;
	final String CONTROL_NOT_EQUAL =	"!=";
	final int NOT_EQUAL = 1;
	final String CONTROL_GREATER_EQUAL =	">=";
	final int GREATER_EQUAL = 2;
	final String CONTROL_GREATER = 		">";
	final int GREATER = 3;
	final String CONTROL_LESS_EQUAL = 	"<=";
	final int LESS_EQUAL = 4;
	final String CONTROL_LESS = 		"<";
	final int LESS = 5;
	
	TextField Status = new TextField();
	TextField Value = new TextField();
	Label Please = new Label();

	Button Color_Now = new Button("Color");
	Button Close = new Button("Close");

	// The colors that it uses
	final Color SELECT_COLOR = Color.blue;
	final Color DESELECT_COLOR = Color.lightGray;
	
	Component display;

	// These will handle all the ioStuff
	int status = 0;
	Event event = new Event(this,Event.ACTION_EVENT,null);

	// Which type of value to use String or Double
	int type;
	final int DOUBLE = 0;
	final int STRING = 1;

        // The current color modal
        static final public int COLOR_HTTP = 0;
        static final public int COLOR_DOMAIN = 1;

	boolean hide;
	boolean in_use = false;
	int http_domain;
	int process;

	public ColorMaskFrame(Component display_input, DList dlist_input, int numFiles, 
				int currentFile, int http_domain_input, String title)
	{
		super (title);

		display = display_input;
		http_domain = http_domain_input;

		switch (http_domain) {
			case COLOR_HTTP:
				Control.addItem(CONTROL_EQUAL);
				Control.addItem(CONTROL_NOT_EQUAL);
				Control.addItem(CONTROL_GREATER_EQUAL);
				Control.addItem(CONTROL_GREATER);
				Control.addItem(CONTROL_LESS_EQUAL);
				Control.addItem(CONTROL_LESS);
				Control.select(CONTROL_GREATER_EQUAL);
				Value.setText("0");
				Please.setText("Type number of HTTP requests");
				break;
			case COLOR_DOMAIN:
				Control.addItem(CONTROL_EQUAL);
				Control.addItem(CONTROL_NOT_EQUAL);
				Please.setText("Type a top level domain name");
				break;
		}

		setList(dlist_input);
	
		sDisplay = new SmallDisplay(dlist,numFiles,currentFile);

		hbar = new Scrollbar(Scrollbar.HORIZONTAL);
		vbar = new Scrollbar(Scrollbar.VERTICAL);

		Process.addItem(PROCESS_COLOR);
		Process.addItem(PROCESS_MASK);
		Process.select(PROCESS_MASK);

		Panel center = new Panel();
		center.setLayout (new BorderLayout());
		center.add("Center", sDisplay);
		center.add("South", hbar);

		Panel west = new Panel();
		GridPanel this_grid = new GridPanel(west);

//		GridPanel this_grid = new GridPanel(this);

		this_grid.addSize(SelectRoot,0,2,2,1);
		this_grid.addSize(Process,0,0,2,1);
		this_grid.addSize(Control,0,3,1,1);

		this_grid.addSize(Value,1,3,1,1);
		this_grid.addSize(Status,0,4,2,1);
		this_grid.addSize(Color_Now,0,5,1,1);
		this_grid.addSize(Close,1,5,1,1);
		
		setLayout(new BorderLayout());
		add("North", Please);
		add("Center", center);
		add("West", west);
		pack();
		resize(400,250);
	}

	public void hide()
	{
		DList mylist = dlist.DList();
		mylist.reset();
		while (!mylist.end())
		{
			DisplayObject object = mylist.next();
			object.show();
			object.setToDefaultColor();
		}
		hide = true;
		display.handleEvent(event);
		super.hide();
	}

	public void setList(DList list)
	{
		dlist = list;
	}

	public boolean isHide()
	{
		return hide;
	}
	
	public boolean inUse()
	{
		return in_use;
	}

	public void inUse(boolean flag)
	{
		in_use = flag;
	}

	public boolean isMask()
	{
		if(process == MASK)
			return true;
		else
			return false;
	} 

	public boolean isHTTP()
	{
		boolean return_value = true;
		switch (http_domain) {
			case COLOR_HTTP:
				return_value = true;
				break; 
			case COLOR_DOMAIN:
				return_value = false;
				break; 
		}
		return return_value;
	}

	public void Color()
	{
		hide = false;

		// --- Setting control
		int control = GREATER_EQUAL;
		String control_string = Control.getSelectedItem();
		if  (control_string.equals(CONTROL_LESS_EQUAL))
			control = LESS_EQUAL;
		else if  (control_string.equals(CONTROL_LESS))
			control = LESS;
		else if  (control_string.equals(CONTROL_GREATER_EQUAL))
			control = GREATER_EQUAL;
		else if  (control_string.equals(CONTROL_GREATER))
			control = GREATER;
		else if  (control_string.equals(CONTROL_NOT_EQUAL))
			control = NOT_EQUAL;
		else if  (control_string.equals(CONTROL_EQUAL))
			control = EQUAL;

		boolean selectRoot = SelectRoot.getState();

		type = DOUBLE;
		if (!isHTTP()) 
			type = STRING;

		process = MASK;
		if (Process.getSelectedItem() == PROCESS_COLOR)
			process = COLOR;
	
		double double_value = 0;
		String string_value = null;
		if (type == DOUBLE)
		{
			try {
				double_value = Double.valueOf(Value.getText())
					.doubleValue();
			} catch (NumberFormatException exception) {
				getToolkit().beep();
				Value.setText("");
				Status.setText("Must be a double");
				System.err.println("ColorByNumber.Color():" +
					exception.toString());
				return;
			}	
		}
		else
		{
			string_value = Value.getText();
		}
			
		int num_matchs = 0;
		DList mylist = dlist.DList();
		DList node_link = new DList();
		mylist.reset();
		boolean match = false;

		while(!mylist.end())
		{
			DisplayObject object = mylist.next();
			object.hide();
			double d_value;
			String s_value;

			if (isHTTP())
			{
				double value = object.getHTTPRequest();
//System.err.println("value: "+ value + "double_value: " + double_value);	
				switch (control)
				{
				     case EQUAL:
					if (value == double_value)
						match = true;
					else
						match = false;
					break;
				     case NOT_EQUAL:
					if (value != double_value)
						match = true;
					else
						match = false;
					break;
				     case GREATER:
					if (value > double_value)
						match = true;
					else
						match = false;
					break;
				     case GREATER_EQUAL:
					if (value >= double_value)
						match = true;
					else
						match = false;
					break;
				     case LESS:
					if (value < double_value)
						match = true;
					else
						match = false;
					break;
				     case LESS_EQUAL:
					if (value <= double_value)
						match = true;
					else
						match = false;
				}
			}
			else 
			{
				if (object instanceof Node) {
					String value = ((Node)object).getDomain();
					if (value.indexOf(string_value) >= 0)
						match = true;
					else
						match = false;

					if (control == NOT_EQUAL)
						match = !match;

					node_link = ((Node)object).getLink();
				}
			}

			if (selectRoot && object instanceof Node)
				((Node)object).setRoot(match);

			if (match)
			{
			  num_matchs++;
			  object.show();
			  if (process == COLOR)
				object.setColor(SELECT_COLOR);
			  if (!isHTTP()&& object instanceof Node) {
				node_link.reset();
				while (!node_link.end()) {
					DisplayObject link =
						node_link.next();
					link.show();
					if (process == COLOR)
						link.setColor(SELECT_COLOR);
				}
			  }
			}
			else
			{
			  if (process == COLOR) {
				object.show();
				object.setColor(DESELECT_COLOR);
			  }
			  if (!isHTTP() && object instanceof Node) {
				node_link.reset();
				while (!node_link.end()) {
					DisplayObject link = node_link.next();
					if (process == COLOR) {
						object.show();
						object.setColor(DESELECT_COLOR);
					}
				}
			  }
			}
		}

		Status.setText("Match: " + num_matchs);
		if (isMask() && isHTTP())
			sDisplay.colorByHTTP();
		if (isMask() && !isHTTP())
			sDisplay.colorByDomain();
		display.handleEvent(event);
	}
	
	public boolean handleEvent(Event event)
	{
		switch (event.id)
		{
			case Event.ACTION_EVENT:
			{
				if (event.target == Close)
					hide();
				else
					Color();
				break;
			}
		}
		if (event.target == hbar)
			 sDisplay.Shift_X( ((Integer)event.arg).intValue());
		if (event.target == vbar)
			 sDisplay.Shift_Y( ((Integer)event.arg).intValue());
		return super.handleEvent(event);
	}
}

class SmallDisplay extends Panel
{
	private final Color BGCOLOR = Color.white;
	private String longest_string = "";

	private int x_shift, y_shift;

        // Attempt to remove redundacny in the system.
        boolean http_colors_set = false;

        // Used by Color Bar
        double max_color_value = 0;
        double min_color_value = 0;
        Color[] colorBarColors = null;
        boolean colorBarUsed = false;
        int COLOR_ANGLE = -120;
        int COLOR_SHIFT = 360;
        final Color COLORBAR_WORD_COLOR = Color.black;

	// The list of domains
        Strings domains;
        Color[] domain_colors;
        boolean colorKeyUsed = false;

        // Attempt to remove redundancy in the system
        // by only seting up the coloring scheme once
        boolean domain_colors_set = false;

	DList dlist;
	int num_files;
	int current_fileIndex;
	
	// Used to hold the offscreen image
	protected Image offscreen;
	protected int offscreen_height;
	protected int offscreen_width;
	protected boolean image_changed;

	public SmallDisplay(DList dlist_input, int numFiles, int currentFile)
	{
		setBackground(BGCOLOR);
		dlist = dlist_input;	
		num_files = numFiles;
		current_fileIndex = currentFile;
		y_shift = x_shift = 0;
		image_changed = true;
	}

        public void colorBarUsed(boolean flag)
        {
                colorBarUsed = flag;
        }

        public void colorKeyUsed(boolean flag)
        {
                colorKeyUsed = flag;
        }

	public void Shift_X(int value) 
	{ 
		x_shift = value; 
		paintBars(getGraphics(),size(),getFontMetrics(getFont()));
	}
	public void Shift_Y(int value) 
	{ 
		y_shift = value; 
		paintBars(getGraphics(),size(),getFontMetrics(getFont()));
	}

	public void paint(Graphics g)
	{
		//Font plain = new Font ("TimesRoman", Font.PLAIN, 16);
		//g.setFont(plain);

		FontMetrics fm = getFontMetrics(getFont());
		Dimension dim = size();
		image_changed = true;

		paintBars(g,dim,fm);
	}

	public void paintBars(Graphics g, Dimension dim, FontMetrics fm)
	{
		if (colorBarUsed && image_changed)
			paintColorBar(dim,fm);
		if (colorKeyUsed && image_changed)
			paintColorKey(dim,fm);
		int x_shift_paint = 0;
		int y_shift_paint = 0;

		if (offscreen_width <= dim.width)
			x_shift_paint = 0;
		else 
			x_shift_paint = (int)
				(x_shift * (dim.width - offscreen_width)/90);

		if (offscreen_height <= dim.height)
			y_shift_paint = 0;
		else 
			y_shift_paint = (int)
				(y_shift * (dim.height - offscreen_height)/90);

		g.clipRect(0,0,dim.width,dim.height);
		g.drawImage(offscreen,x_shift_paint,y_shift_paint,this);
		g.setColor(getBackground());
		if (x_shift_paint >0)
			g.fillRect(0,0,x_shift_paint,dim.height);
		if (y_shift_paint >0)
			g.fillRect(0,0,dim.width,y_shift_paint);
		if (dim.width > offscreen_width + x_shift_paint)
		{
			int erase_x = offscreen_width + x_shift_paint;
			int erase_width = dim.width - erase_x;
			g.fillRect(erase_x,0,erase_width,dim.height);
		}
		if (dim.height > offscreen_height + y_shift_paint)
		{ 
			int erase_y = offscreen_height + y_shift_paint;
			int erase_height = dim.height - erase_y;
			g.fillRect(0,erase_y,dim.width,erase_height);
		}
	}
	
	public void colorByDomain()
	{
		DList mylist = dlist.DList();
		if (!domain_colors_set)
		{
			domains = new Strings();
			mylist.reset();
			while(!mylist.end())
			{
			    DisplayObject object = mylist.next();
			    if (object instanceof Node)
			    {
			    	for(int i=0; i < num_files; i++)
				{
					String domain = ((Node)object)
						.getDomain();
					domains.addUnique(domain);
				}
			    }
			}
			domains.sort();
			int length = domains.length();
			domain_colors = createSpectrum(length);
			mylist.reset();
			while(!mylist.end())
			{
			    DisplayObject object = mylist.next();	
			    if (object instanceof Node)
			    {
				for(int i=0; i < num_files; i++)
				{
					object.setFileIndex(i);
					String domain = ((Node)object)
						.getDomain();
					int index = domains.indexOf(domain);
					object.setDomainColor(
						domain_colors[index]);
				}
			   	object.setFileIndex(current_fileIndex);
			   	object.setToDomainColor();
			    }
			    else
				object.setToDefaultColor();
			}
			domain_colors_set = true;
		}
		else
		{
			mylist.reset();
			while(!mylist.end())
			{
				DisplayObject object = mylist.next();
				if (object instanceof Node)
					object.setToDomainColor();
				else
					object.setToDefaultColor();
			}
		}
		colorBarUsed(false);
		colorKeyUsed(true);
		repaint();
	}
			
	/********************************************************
	* Purpose:To color all objects by there HTTP
	*********************************************************/
	public void colorByHTTP()
	{
		DList mylist = dlist.DList();
		if (!http_colors_set)
		{
			max_color_value = 0;
			min_color_value = 0;

			mylist.reset();
			while(!mylist.end())
			{
			    DisplayObject object = mylist.next();
			    for(int i=0; i < num_files; i++)
			    {
				object.setFileIndex(i);
				if (object.exists())
				{
					double value = Math.log( 
						object.getHTTPRequest());
					if (value < min_color_value)
						min_color_value = value;
					if (value > max_color_value)
						max_color_value = value;
				}
			    }
			}
			
			mylist.reset();
			while(!mylist.end())	
			{
			    DisplayObject object = mylist.next();
			    for(int i=0; i < num_files; i++)
			    {
				object.setFileIndex(i);
				if (object.exists())
				{
			 	  double value = Math.log(
					object.getHTTPRequest());
				  double hue = COLOR_ANGLE*(1-(value-min_color_value)
				    / (max_color_value-min_color_value)) + COLOR_SHIFT;
				  object.setHTTPColor(HueColor((float)hue));
				}
			   }
			   object.setFileIndex(current_fileIndex);
			   object.setToHTTPColor();
			} 
			http_colors_set = true;
			colorBarColors = null;
		}
		else
		{
			mylist.reset();
			while(!mylist.end())
				mylist.next().setToHTTPColor();	
		}
		colorKeyUsed(false);
		colorBarUsed(true);
		repaint();
	}
	
	protected void paintColorBar(Dimension dim, FontMetrics fm)
	{
		int old_offscreen_width = offscreen_width;
		int old_offscreen_height = offscreen_height;
		CheckOffscreenSizeBar(dim,fm);

		if (offscreen == null || old_offscreen_width != offscreen_width || old_offscreen_height != offscreen_height)
			offscreen = createImage(offscreen_width,offscreen_height);
		Graphics g = offscreen.getGraphics();
		g.setColor(getBackground());
		g.fillRect(0,0,offscreen_width,offscreen_height);

		int charHeight = fm.getHeight();
		int barX = fm.charWidth(' ');
		int barX2 = 2*barX + barX;
		int topY = dim.height/9;
		int barY = topY;
		int lastvalue = 1;

		int barHeight = dim.height - 2*topY;

		int index=0;
		g.setColor(COLORBAR_WORD_COLOR);
		g.drawString("HTTP Requests",barX,barY);
		barY += charHeight/2;
		while (index < colorBarColors.length)
		{
			g.setColor(colorBarColors[index]);
			g.drawLine(barX,barY,barX2,barY);
			barY++;
			index++;
		}
		
		int bar_bottom = barY = topY + charHeight/2;
		int bar_top = barY + colorBarColors.length;
		barX2 += barX/2;
		while (barY <= bar_top)
		{
			g.setColor(COLORBAR_WORD_COLOR);
			g.drawLine(barX,barY,barX2,barY);
			barY += charHeight/2;
			double value = ((max_color_value-min_color_value)
				* (barY-bar_bottom)/(bar_top - bar_bottom)
				 +min_color_value);
			value = Math.exp(value);
			g.setColor(COLORBAR_WORD_COLOR);
			g.drawString (String.valueOf((int)value),barX2, barY);
			barY += charHeight;
			lastvalue = (int)value;
		}
		barY -= charHeight/2;
		if (barY <= bar_top)
		{
			g.setColor(COLORBAR_WORD_COLOR);
			g.drawLine(barX,barY,barX2,barY);
			barY += charHeight/2;
			double value = ((max_color_value-min_color_value)
				* (barY-bar_bottom)/(bar_top - bar_bottom)
				 +min_color_value);
			value = Math.exp(value);
			g.setColor(COLORBAR_WORD_COLOR);
			g.drawString (String.valueOf((int)value),barX2, barY);
			lastvalue = (int)value;
		}
	}

	protected void CheckOffscreenSizeBar(Dimension dim, FontMetrics fm)
	{
		int charHeight = fm.getHeight();
		int barX = fm.charWidth(' ');
		int barX2 = 2*barX + barX;
		int topY = dim.height/9;
		int barY = topY;
		int lastvalue = 1;

		int barHeight = dim.height - 2*topY;

		if (colorBarColors == null 
			|| colorBarColors.length != barHeight)
		{
			colorBarColors = new Color[barHeight];
			for (int i=0; i < colorBarColors.length;i++)
			{
				float hue = 
				  COLOR_ANGLE*(1-((float)(i)/
				  colorBarColors.length)) + COLOR_SHIFT;
				colorBarColors[i] = HueColor(hue);
			}
		}

		int index=0;
		barY += charHeight/2;
		while (index < colorBarColors.length)
		{
			barY++;
			index++;
		}
		
		int bar_bottom = barY = topY + charHeight/2;
		int bar_top = barY + colorBarColors.length;
		barX2 += barX/2;
		while (barY <= bar_top)
		{
			barY += charHeight/2;
			double value = ((max_color_value-min_color_value)
				* (barY-bar_bottom)/(bar_top - bar_bottom)
				 +min_color_value);
			value = Math.exp(value);
			barY += charHeight;
			lastvalue = (int)value;
		}
		barY -= charHeight/2;
		if (barY <= bar_top)
		{
			barY += charHeight/2;
			double value = ((max_color_value-min_color_value)
				* (barY-bar_bottom)/(bar_top - bar_bottom)
				 +min_color_value);
			value = Math.exp(value);
			lastvalue = (int)value;
		}
		offscreen_height = barY;
		int title = fm.stringWidth("HTTP Requests");
		int last = fm.stringWidth(String.valueOf((int)lastvalue));
		int length = 0;
		length = (title > last) ? title : last;
		offscreen_width = barX2 + length ;
	}

	protected void paintColorKey(Dimension dim, FontMetrics fm)
	{
		int old_offscreen_width = offscreen_width;
		int old_offscreen_height = offscreen_height;
		CheckOffscreenSizeKey(dim,fm);

		if (offscreen == null || old_offscreen_width != offscreen_width || old_offscreen_height != offscreen_height)
			offscreen = createImage(offscreen_width,offscreen_height);

		Graphics g = offscreen.getGraphics();
		g.setColor(getBackground());
		g.fillRect(0,0,offscreen_width,offscreen_height);

		int charHeight = fm.getHeight();
		int charWidth = fm.charWidth(' ');
		int barX = charWidth;
		int stringX = barX + (3*charWidth)/2;
		int topY = dim.height/9;
		int barY = topY;
		int max_height  = dim.height - topY;
		g.setColor(COLORBAR_WORD_COLOR);
		g.drawString("Domain Names",barX, barY);
		barY += charHeight/2;
		int longest = 0;
		for(int index=0; index < domain_colors.length; index++)
		{
			g.setColor(domain_colors[index]);
			g.fillRect(barX,barY,charWidth,charHeight);
			barY += charHeight;
			g.setColor(COLORBAR_WORD_COLOR);
			String domain = domains.get(index);
			int length = fm.stringWidth(domain);
			if (length > longest)
				longest = length;
			g.drawString (domain,stringX,barY);
			barY += charHeight/2;
			if (barY >=max_height)
			{
				//barY = charHeight;
				barY = topY + charHeight/2;
				barX +=  longest + 2*charWidth;
				stringX += longest + 2*charWidth;
				longest = 0;
			}
		}
	}

	protected void CheckOffscreenSizeKey(Dimension dim, FontMetrics fm)
	{
		int charHeight = fm.getHeight();
		int charWidth = fm.charWidth(' ');
		int barX = charWidth;
		int stringX = barX + (3*charWidth)/2;
		int topY = dim.height/9;
		int barY = topY;
		int max_height  = dim.height - topY;
		barY += charHeight/2;
		int longest = 0;
		for(int index=0; index < domain_colors.length; index++)
		{
			barY += charHeight;
			String domain = domains.get(index);
			int length = fm.stringWidth(domain);
			if (length > longest)
				longest = length;
			barY += charHeight/2;
			if (barY >=max_height)
			{
				barY = topY + charHeight/2;
				barX +=  longest + 2*charWidth;
				stringX += longest + 2*charWidth;
				longest = 0;
			}
		}
		offscreen_height = (max_height > barY) ? max_height : barY;
		// give some redundant space
		offscreen_height += charHeight;
		offscreen_width = stringX + longest + 2*charWidth;
	}

	protected static Color[] createSpectrum(int length)
        {
                Color[] colors = new Color[length];
                for (int i=0;i < length; i++)
                {
                        float hue = (300*i)
                                /length;
                        colors[i] = HueColor(hue);
                }
                return  colors;
        }

        protected static Color HueColor(float input)
        {
                input = input / 60;
                while (input >= 6) input -= 6;
                while (input <  0) input += 6;
                int i = (int) input;
                float v = 1;
                float f = input - i;
                float p = 0;
                        float q = 1-f;
                float red=1,green=1,blue=1;
                switch (i)
                {
                        case 0: red=v; green=f; blue=p; break;
                        case 1: red=q; green=v; blue=p; break;
                        case 2: red=p; green=v; blue=f; break;
                        case 3: red=f; green=q; blue=v; break;
                        case 4: red=f; green=p; blue=v; break;
                        case 5: red=v; green=p; blue=q; break;
                }

                return new Color(red,green,blue);
        }
}
